@@ -33,7 +33,7 @@ Some improvement and bug fixes available in newer version of Mercurial are also
 backported to older version of Mercurial by this extension. Some older
 experimental protocols are also supported for a longer time in the extension to
 help people transitioning. (The extension is currently compatible down to
-Mercurial version 4.7).
+Mercurial version 4.8).
 
 New Config::
 
@@ -1151,6 +1151,11 @@ def _evolvemessage():
              b'(also see `hg help evolve.interrupted`)')
     return cmdutil._commentlines(_msg)
 
+def _fixupmessage():
+    _msg = _(b'To continue:    hg fixup --continue\n'
+             b'To abort:       hg fixup --abort\n')
+    return cmdutil._commentlines(_msg)
+
 @eh.uisetup
 def setupevolveunfinished(ui):
     if not util.safehasattr(cmdutil, 'unfinishedstates'):
@@ -1165,6 +1170,10 @@ def setupevolveunfinished(ui):
                                abortfunc=evolvecmd.hgabortevolve)
         statemod.addunfinished(b'pick', fname=b'pickstate', continueflag=True,
                                abortfunc=cmdrewrite.hgabortpick)
+        _fixup_msg = _(b'To continue:    hg fixup --continue\n'
+                       b'To abort:       hg fixup --abort\n')
+        statemod.addunfinished(b'fixup', fname=b'fixup-state',
+                               continueflag=True, statushint=_fixup_msg)
     else:
         # hg <= 5.0 (5f2f6912c9e6)
         estate = (b'evolvestate', False, False, _(b'evolve in progress'),
@@ -1173,16 +1182,23 @@ def setupevolveunfinished(ui):
         pstate = (b'pickstate', False, False, _(b'pick in progress'),
                   _(b"use 'hg pick --continue' or 'hg pick --abort' to abort"))
         cmdutil.unfinishedstates.append(pstate)
+        fstate = (b'fixup-state', False, False, _(b'fixup in progress'),
+                  _(b"use 'hg fixup --continue' or 'hg fixup --abort' to abort"))
+        cmdutil.unfinishedstates.append(fstate)
 
         afterresolved = (b'evolvestate', _(b'hg evolve --continue'))
         pickresolved = (b'pickstate', _(b'hg pick --continue'))
+        fixupresolved = (b'fixup-state', _(b'hg fixup --continue'))
         cmdutil.afterresolvedstates.append(afterresolved)
         cmdutil.afterresolvedstates.append(pickresolved)
+        cmdutil.afterresolvedstates.append(fixupresolved)
 
+    # hg <= 5.0 (12243f15d53e)
     if util.safehasattr(cmdutil, 'STATES'):
-        statedata = (b'evolve', cmdutil.fileexistspredicate(b'evolvestate'),
-                     _evolvemessage)
-        cmdutil.STATES = (statedata, ) + cmdutil.STATES
+        cmdutil.STATES = (
+            (b'evolve', cmdutil.fileexistspredicate(b'evolvestate'), _evolvemessage),
+            (b'fixup', cmdutil.fileexistspredicate(b'fixup-state'), _fixupmessage),
+        ) + cmdutil.STATES
 
 @eh.wrapfunction(hg, 'clean')
 def clean(orig, repo, *args, **kwargs):
