@@ -30,6 +30,7 @@ from mercurial import (
     pycompat,
     scmutil,
     util,
+    repair,
 )
 
 from mercurial.utils import dateutil
@@ -42,6 +43,7 @@ from . import (
     exthelper,
     rewriteutil,
     utility,
+    evolvecmd,
 )
 
 eh = exthelper.exthelper()
@@ -1430,10 +1432,11 @@ def abortpick(ui, repo, pickstate, abortcmd=False):
         raise error.Abort(_(b"no interrupted pick state exists"))
     pickstate.load()
     pctxnode = pickstate[b'oldpctx']
-    ui.status(_(b"aborting pick, updating to %s\n") %
-              node.short(pctxnode))
     compat.clean_update(repo[pctxnode])
     pickstate.delete()
+    ui.status(_(b'pick aborted\n'))
+    ui.status(_(b'working directory is now at %s\n')
+              % node.short(pctxnode))
     return 0
 
 def hgabortpick(ui, repo):
@@ -1441,3 +1444,153 @@ def hgabortpick(ui, repo):
     with repo.wlock(), repo.lock():
         pickstate = state.cmdstate(repo, path=b'pickstate')
         return abortpick(ui, repo, pickstate, abortcmd=True)
+
+@eh.command(
+    b'fixup|fix-up',
+    [
+        (b'r', b'rev', b'', _(b'revision to amend'), _(b'REV')),
+        (b'c', b'continue', False, _(b'continue an interrupted fixup')),
+        (b'', b'abort', False, _(b'abort an interrupted fixup')),
+    ],
+    _(b'[OPTION]... [-r] REV'),
+    helpbasic=True,
+    **compat.helpcategorykwargs('CATEGORY_COMMITTING')
+)
+def fixup(ui, repo, node=None, **opts):
+    """add working directory changes to an arbitrary revision
+
+    A new changeset will be created, superseding the one specified. The new
+    changeset will combine working directory changes with the changes in the
+    target revision.
+
+    This operation requires the working directory changes to be relocated onto
+    the target revision, which might result in merge conflicts.
+
+    If fixup is interrupted to manually resolve a conflict, it can be continued
+    with --continue/-c, or aborted with --abort.
+
+    Note that this command is fairly new and its behavior is still
+    experimental. For example, the working copy will be left on a temporary,
+    obsolete commit containing the fixed-up changes after the operation. This
+    might change in the future.
+
+    Returns 0 on success, 1 if nothing changed.
+    """
+    compat.check_at_most_one_arg(opts, 'continue', 'abort')
+    with repo.wlock(), repo.lock():
+        return _perform_fixup(ui, repo, node, **opts)
+
+def _perform_fixup(ui, repo, node, **opts):
+    contopt = opts.get('continue')
+    abortopt = opts.get('abort')
+    if node or opts.get('rev'):
+        if contopt:
+            raise error.Abort(_(b'cannot specify a revision with --continue'))
+        if abortopt:
+            raise error.Abort(_(b'cannot specify a revision with --abort'))
+    # state file for --continue/--abort cases
+    fixup_state = state.cmdstate(repo, b'fixup-state')
+    if contopt:
+        if not fixup_state.exists():
+            raise error.Abort(_(b'no interrupted fixup to continue'))
+        fixup_state.load()
+        return continuefixup(ui, repo, fixup_state)
+    if abortopt:
+        if not fixup_state.exists():
+            raise error.Abort(_(b'no interrupted fixup to abort'))
+        fixup_state.load()
+        return abortfixup(ui, repo, fixup_state)
+
+    if node and opts.get('rev'):
+        raise error.Abort(_(b'please specify just one revision'))
+    if not node:
+        node = opts.get('rev')
+    if not node:
+        raise error.Abort(_(b'please specify a revision to fixup'))
+    target_ctx = scmutil.revsingle(repo, node)
+
+    fixup_state[b'startnode'] = repo[b'.'].node()
+
+    tr = repo.transaction(b'fixup')
+    with util.acceptintervention(tr):
+        overrides = {(b'ui', b'allowemptycommit'): False}
+        with repo.ui.configoverride(overrides, b'fixup'):
+            tempnode = repo.commit(
+                text=b'temporary fixup commit', user=opts.get(b'user'),
+                date=opts.get(b'date'))
+            if tempnode is None:
+                ui.status(_(b"nothing changed\n"))
+                return 1
+        fixup_state[b'tempnode'] = tempnode
+        # XXX: storing 'tempnode' should be enough, but 'current'
+        # is used by _relocate() logic
+        fixup_state[b'current'] = tempnode
+        fixup_state[b'target'] = target_ctx.node()
+        with state.saver(fixup_state):
+            # relocate temporary node to target revision
+            newnode = evolvecmd._relocate(
+                repo, repo[tempnode], target_ctx, fixup_state, update=False
+            )
+        # fold the two changesets
+        revs = (repo[newnode].rev(), target_ctx.rev())
+        root, head, p2 = rewriteutil.foldcheck(repo, revs)
+
+        allctx = [repo[r] for r in revs]
+        commitopts = {b'edit': False, b'message': target_ctx.description()}
+        newid, unusedvariable = rewriteutil.rewrite(
+            repo, root, head, [root.p1().node(), p2.node()],
+            commitopts=commitopts
+        )
+        phases.retractboundary(repo, tr, target_ctx.phase(), [newid])
+        replacements = {tuple(ctx.node() for ctx in allctx): [newid]}
+        compat.cleanupnodes(repo, replacements, operation=b'fixup')
+        fixup_state.delete()
+        compat.update(repo.unfiltered()[tempnode])
+        return 0
+
+def continuefixup(ui, repo, fixup_state):
+    """logic for handling of `hg fixup --continue`"""
+    target_node = fixup_state[b'target']
+    tempnode = fixup_state[b'tempnode']
+    target_ctx = repo[target_node]
+    tr = repo.transaction(b'fixup')
+    with util.acceptintervention(tr):
+        newnode = evolvecmd._completerelocation(ui, repo, fixup_state)
+        current = repo[fixup_state[b'current']]
+        obsolete.createmarkers(repo, [(current, (repo[newnode],))],
+                               operation=b'fixup')
+        # fold the two changesets
+        revs = (repo[newnode].rev(), target_ctx.rev())
+        root, head, p2 = rewriteutil.foldcheck(repo, revs)
+
+        allctx = [repo[r] for r in revs]
+        commitopts = {b'edit': False, b'message': target_ctx.description()}
+        newid, unusedvariable = rewriteutil.rewrite(
+            repo, root, head, [root.p1().node(), p2.node()],
+            commitopts=commitopts
+        )
+        phases.retractboundary(repo, tr, target_ctx.phase(), [newid])
+        replacements = {tuple(ctx.node() for ctx in allctx): [newid]}
+        compat.cleanupnodes(repo, replacements, operation=b'fixup')
+        fixup_state.delete()
+        compat.update(repo.unfiltered()[tempnode])
+        return 0
+
+def abortfixup(ui, repo, fixup_state):
+    """logic for handling of `hg fixup --abort`"""
+    with repo.wlock(), repo.lock():
+        startnode = fixup_state[b'startnode']
+        tempnode = fixup_state[b'tempnode']
+        tempctx = repo[tempnode]
+        compat.clean_update(repo[startnode])
+
+        stats = merge.graft(repo, tempctx, tempctx.p1(), [b'graft', b'fixup'])
+        # conflict is not possible, since grafting changes from descendant
+        assert not stats.unresolvedcount
+        repair.strip(ui, repo, [tempnode], backup=False)
+
+    pctx = repo[b'.']
+    ui.status(_(b'fixup aborted\n'))
+    ui.status(_(b'working directory is now at %s\n') % pctx)
+    fixup_state.delete()
+    return 0
