@@ -133,14 +133,14 @@ def _solveorphan(ui, repo, orig, evolvestate, display, dryrun=False,
             ui.write_err(msg)
             return (False, b".")
         if exc.splitflag:
-            splitsucc = utility.picksplitsuccessor(ui, repo, obs, orig)
-            if not splitsucc[0]:
+            splitsucc = utility.select_split_successor(ui, repo, obs)
+            if not splitsucc:
                 msg = _(b"could not solve instability, "
                         b"ambiguous destination: "
                         b"parent split across two branches\n")
                 ui.write_err(msg)
                 return (False, b".")
-            newer = splitsucc[1]
+            newer = splitsucc
     target = repo[newer]
     if not ui.quiet or confirm:
         repo.ui.write(_(b'move:'), label=b'evolve.operation')
@@ -816,16 +816,18 @@ def mergecommitmessages(ui, basedesc, divdesc, othdesc):
     there were conflicts or not while merging the messages"""
 
     merger = simplemerge.Merge3Text(basedesc, divdesc, othdesc)
-    mdesc = []
-    kwargs = {}
-    kwargs['name_base'] = b'base'
-    kwargs['base_marker'] = b'|||||||'
-    for line in merger.merge_lines(name_a=b'divergent', name_b=b'other',
-                                   **kwargs):
-        mdesc.append(line)
+    kwargs = {'name_a': b'divergent', 'name_b': b'other', 'name_base': b'base'}
 
-    desc = b''.join(mdesc)
-    if merger.conflicts:
+    if util.safehasattr(simplemerge, 'render_merge3'):
+        lines, conflicts = simplemerge.render_merge3(merger, **kwargs)
+        desc = b''.join(lines)
+    else:
+        # hg <= 6.0 (12ac4401ff7d)
+        kwargs['base_marker'] = b'|||||||'
+        desc = b''.join(merger.merge_lines(**kwargs))
+        conflicts = merger.conflicts
+
+    if conflicts:
 
         prefixes = (b"HG: Conflicts while merging changeset description of"
                     b" content-divergent changesets.\nHG: Resolve conflicts"
@@ -1138,18 +1140,14 @@ def _sorted_divergents(repo, divergent_revs):
     Fall back to rev number, but in ascending order, for historical reasons.
     """
     ledmap = {
-        # For purpose of comparing, from `led` which is (unixtime, offset)
-        # we only need `unixtime`.
+        # for comparing, we only need unixtime from (unixtime, offset)
         rev: latest_evolution_date(repo, repo[rev])[0]
         for rev in divergent_revs
     }
-    # Sorting by negating the `rev` in key func, to fallback to the old way
-    # of selecting revision, in case when `led` is same while comparing.
-    # Old way: was to select the one with minimum revision number
     return sorted(divergent_revs, key=lambda rev: (-ledmap[rev], rev))
 
 def latest_evolution_date(repo, ctx):
-    """Return latest evolution date of `ctx`"""
+    """Return latest evolution date of `ctx` as (unixtime, offset)"""
     node = ctx.node()
     nodes = list(obsutil.closestpredecessors(repo, node))
     nodes.append(node)
@@ -1814,21 +1812,33 @@ def _solveonerev(ui, repo, rev, evolvestate, activetopic, dryrunopt, confirmopt,
     return ret
 
 def solveobswdp(ui, repo, opts):
-    """this function updates to the successor of obsolete wdir parent"""
+    """update to the successor of obsolete wdir parent"""
     oldid = repo[b'.'].node()
-    startctx = repo[b'.']
+    oldctx = repo[b'.']
     dryrunopt = opts.get('dry_run', False)
     display = compat.format_changeset_summary_fn(ui, repo, b'evolve',
                                                  shorttemplate)
     try:
-        ctx = repo[utility._singlesuccessor(repo, repo[b'.'])]
+        ctx = repo[utility._singlesuccessor(repo, oldctx)]
     except utility.MultipleSuccessorsError as exc:
-        repo.ui.write_err(_(b'parent is obsolete with multiple'
-                            b' successors:\n'))
-        for ln in exc.successorssets:
-            for n in ln:
-                display(repo[n])
-        return 2
+        if exc.splitflag:
+            splitsucc = utility.select_split_successor(ui, repo, oldctx)
+            if splitsucc:
+                ctx = repo[splitsucc]
+            else:
+                if util.safehasattr(error, 'CanceledError'):
+                    err = error.CanceledError
+                else:
+                    # hg <= 5.6 (ac362d5a7893)
+                    err = error.Abort
+                raise err(_(b'user quit'))
+        elif exc.divergenceflag:
+            repo.ui.write_err(_(b'parent is obsolete with multiple'
+                                b' content-divergent successors:\n'))
+            for sset in exc.successorssets:
+                for succ in sset:
+                    display(repo[succ])
+            return 2
 
     ui.status(_(b'update:'))
     if not ui.quiet:
@@ -1839,7 +1849,7 @@ def solveobswdp(ui, repo, opts):
     res = hg.update(repo, ctx.rev())
     newid = ctx.node()
 
-    if ctx != startctx:
+    if ctx != oldctx:
         with repo.wlock(), repo.lock(), repo.transaction(b'evolve') as tr:
             bmupdater = rewriteutil.bookmarksupdater(repo, oldid, tr)
             bmupdater(newid)
