@@ -92,9 +92,9 @@ from mercurial import (
 
 from mercurial.i18n import _
 
-__version__ = b'0.1.3.dev'
-testedwith = b'4.7 4.8 4.9 5.0 5.1 5.2 5.3 5.4 5.5 5.6 5.7 5.8 5.9'
-minimumhgversion = b'4.7'
+__version__ = b'0.1.3'
+testedwith = b'4.8 4.9 5.0 5.1 5.2 5.3 5.4 5.5 5.6 5.7 5.8 5.9 6.0 6.1'
+minimumhgversion = b'4.8'
 buglink = b'https://bz.mercurial-scm.org/'
 
 cmdtable = {}
